﻿using System;
using System.Data;
using System.Drawing.Imaging;
using System.IO;
using System.Transactions;
using System.Web.Mvc;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki.MVC
{
    public class ImageController : ShuttleController
    {
        public static string Name = "Image";
        private readonly IWikiImageRepository repository;
        private readonly IWikiImageQuery query;

        public ImageController(IWikiImageRepository repository, IWikiImageQuery query)
        {
            this.repository = repository;
            this.query = query;
        }

        public IImageService ImageService { get; set; }

        [HttpGet]
        public FileResult Raw()
        {
            DataRow row;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                row = query.GetForRendering(RouteData.Id()).Row();
            }

            if (row == null)
            {
                return null;
            }

            var path = Path.Combine(ApplicationConfiguration.MediaFolder,
                                    string.Format(@"{0}.{1}", WikiImageColumns.Id.MapFrom(row).ToString("n"),
                                                  WikiImageColumns.FileExtension.MapFrom(row)));

            return new FilePathResult(path, "application/octet-stream");
        }

        [HttpGet]
        [MemberAccess]
        public ActionResult Manage()
        {
            return View();
        }

        [HttpGet]
        [MemberAccess]
        public ActionResult New()
        {
            return View(TempData.Get<ImageModel>()
                        ?? new ImageModel(Actions.Add, Name));
        }

        [HttpGet]
        [MemberAccess]
        public ActionResult Edit()
        {
            var id = RouteData.IdAsGuid();

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                return View(TempData.Get<ImageModel>()
                            ?? new ImageModel(Actions.Save, Name)
                                   {
                                       Id = id,
                                       Name = WikiImageColumns.Name.MapFrom(query.GetName(id).Row())
                                   });
            }
        }

        [HttpPost]
        [MemberAccess]
        public ActionResult Add()
        {
            var name = ImageView.ImageName.MapFrom(Request.Form);

            if (Request.Files.Count == 0)
            {
                RegisterFailureMessage("No file has been selected for upload.");

                return ValidationFailureAdd(name);
            }

            var file = Request.Files[0];

            if (file == null || file.ContentLength == 0)
            {
                RegisterFailureMessage("No file has been selected for upload or the file is empty.");

                return ValidationFailureAdd(name);
            }

            var format = ImageService.ImageFormat(file.InputStream);

            if (format != ImageFormat.Gif && format != ImageFormat.Jpeg && format != ImageFormat.Png)
            {
                RegisterFailureMessage("Only gif, jpg and png files are supported.");

                return ValidationFailureAdd(name);
            }

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                if (query.HasName(name))
                {
                    RegisterFailureMessage(string.Format("Image with name '{0}' already exists.", name));

                    return ValidationFailureAdd(name);
                }
            }

            using (var scope = new TransactionScope())
            {
                var id = Guid.NewGuid();

                var fileExtension = ImageService.FileExtension(format);

                var path = Path.Combine(ApplicationConfiguration.MediaFolder,
                                        string.Format("{0}.{1}", id.ToString("n"), fileExtension));

                if (System.IO.File.Exists(path))
                {
                    System.IO.File.Delete(path);
                }

                file.SaveAs(path);

                Bus.Send(new AddWikiImageCommand
                             {
                                 Id = id,
                                 Name = name,
                                 FileExtension = fileExtension
                             });

                scope.Complete();
            }

            return RedirectToAction(Actions.Manage);
        }

        [HttpPost]
        [MemberAccess]
        public ActionResult Save()
        {
            WikiImage image;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                image = repository.Get(RouteData.IdAsGuid());
            }

            if (image == null)
            {
                RegisterFailureMessage("Image '{0}' no longer exists.");

                RedirectToAction(Actions.Manage);
            }

            var name = ImageView.ImageName.MapFrom(Request.Form);

            var fileExtension = string.Empty;

            if (Request.Files.Count == 1 && Request.Files[0] != null && Request.Files[0].ContentLength > 0)
            {
                var existingPath = Path.Combine(ApplicationConfiguration.MediaFolder,
                                                string.Format("{0}.{1}", image.Id.ToString("n"), image.FileExtension));

                if (System.IO.File.Exists(existingPath))
                {
                    System.IO.File.Delete(existingPath);
                }

                var file = Request.Files[0];

                var format = ImageService.ImageFormat(file.InputStream);

                fileExtension = ImageService.FileExtension(format);

                if (format != ImageFormat.Gif && format != ImageFormat.Jpeg && format != ImageFormat.Png)
                {
                    RegisterFailureMessage("Only gif, jpg and png files are supported.");

                    TempData.Add(new ImageModel(Actions.Add, Name)
                                     {
                                         Name = name
                                     });

                    return RedirectToAction(Actions.Edit);
                }

                var path = Path.Combine(ApplicationConfiguration.MediaFolder,
                                        string.Format("{0}.{1}", image.Id.ToString("n"), fileExtension));

                if (System.IO.File.Exists(path))
                {
                    System.IO.File.Delete(path);
                }

                file.SaveAs(path);
            }

            using (var scope = new TransactionScope())
            {
                Bus.Send(new SaveWikiImageCommand
                             {
                                 Id = image.Id,
                                 Name = name,
                                 FileExtension = fileExtension
                             });

                scope.Complete();
            }

            return RedirectToAction(Actions.Manage);
        }

        private ActionResult ValidationFailureAdd(string name)
        {
            TempData.Add(new ImageModel(Actions.Add, Name)
                             {
                                 Name = name
                             });

            return RedirectToAction(Actions.New);
        }

        [HttpPost]
        [MemberAccess]
        [Ajax]
        public ActionResult List(string name)
        {
            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                return Json(new
                                {
                                    success = true,
                                    view = this.RenderPartialViewToString("List", query.Search(name))
                                });
            }
        }

        public static class Actions
        {
            public static readonly string Add = "Add";
            public static readonly string Edit = "Edit";
            public static readonly string Manage = "Manage";
            public static readonly string New = "New";
            public static readonly string Render = "Render";
            public static readonly string Save = "Save";
            public static readonly string List = "List";
            public static readonly string ShowProperties = "ShowProperties";
        }
    }
}