﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net.Mail;
using System.Web.Mvc;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki.MVC
{
    public class MemberController : ShuttleController
    {
        public static string Name = "Member";

        private readonly IMemberActivationTask memberActivationTask;
        private readonly IMemberLoginTask memberLoginTask;
        private readonly IMemberRepository repository;
        private readonly IMemberQuery query;

        public IEMailGateway EMailGateway { get; set; }
        public ICryptographyService CryptographyService { get; set; }

        public MemberController(IMemberActivationTask memberActivationTask, IMemberLoginTask memberLoginTask, IMemberRepository repository, IMemberQuery query)
        {
            this.memberActivationTask = memberActivationTask;
            this.memberLoginTask = memberLoginTask;
            this.repository = repository;
            this.query = query;
        }

        [HttpGet]
        public ActionResult Credentials()
        {
            return View(new CredentialsModel(Actions.Login, Name)
                            {
                                Identifier = Request.Params["Name"] ?? string.Empty,
                                SubmitButtonText = "Login"
                            });
        }

        [HttpGet]
        public ActionResult CredentialsReminder()
        {
            return View(new FormModel(Actions.SendCredentials, Name)
                            {
                                SubmitButtonText = "Send Credentials"
                            });
        }

        [HttpPost]
        public ActionResult SendCredentials()
        {
            Member member;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                member = repository.GetByEMailAddress(CredentialsReminderView.EMailAddress.MapFrom(Request.Form));

                if (member == null)
                {
                    RegisterFailureMessage("The specified e-mail address does not exist.");

                    return RedirectToAction(Actions.CredentialsReminder);
                }

                memberActivationTask.SendCredentialsReminder(member);
            }

            RegisterSuccessMessage("Credentials reminder has been sent.");

            return WikiIndex();
        }

        [HttpPost]
        public ActionResult Login()
        {
            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                var memberName = CredentialsView.MemberName.MapFrom(Request.Form);

                switch (memberLoginTask.LogIn(memberName, CredentialsView.Password.MapFrom(Request.Form)))
                {
                    case MemberLoginResult.MemberNotFound:
                    case MemberLoginResult.PasswordMismatch:
                        {
                            RegisterFailureMessage("Incorrect member name or password.");

                            return RedirectToAction(Actions.Credentials);
                        }
                    case MemberLoginResult.MemberNotYetActivated:
                        {
                            memberActivationTask.SendActivationEMail(repository.GetByName(memberName));

                            RegisterFailureMessage("Your account has not yet been activated by clicking on the activation link in the e-mail that was sent to you.");
                            RegisterFailureMessage(string.Format("Please check your e-mail and if necessary whitelist our domain: {0}", SystemSettings().EMailDomain));
                            RegisterFailureMessage("The activation e-mail has been sent to you again.");

                            return RedirectToAction(Actions.Credentials);
                        }
                }
            }

            return WikiIndex();
        }

        [HttpGet]
        public ActionResult Logout()
        {
            UserSession.LogOff();

            return WikiIndex();
        }

        [HttpGet]
        public ActionResult EditEMailAddress()
        {
            string emailAddress;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                emailAddress = query.EMailAddress(UserSession.MemberId);
            }

            return View(Models.EMailModel(emailAddress));
        }

        [HttpPost]
        public ActionResult ApplyNewEMailAddress()
        {
            var model = Models.EMailModel(MemberView.EMailAddress.MapFrom(Request.Form));

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                var member = repository.Get(UserSession.MemberId);

                Guard.AgainstMissing<Member>(member, UserSession.MemberId);

                member.NewEMailAddress = new EMailAddress(model.EMailAddress);

                if (member.EMailAddressesAreTheSame)
                {
                    RegisterSuccessMessage("Your e-mail address has not changed.");

                    return RedirectToAction(Actions.Profile);
                }

                repository.SaveNewEMailAddress(member);

                memberActivationTask.SendNewEMailAddressEMail(member);
            }

            RegisterSuccessMessage("An e-mail has been sent to the address you just registered.  Please click on the activation link in the e-mail to apply the new address.");

            return RedirectToAction(Actions.Profile);
        }


        [HttpGet]
        public ActionResult Register()
        {
            if (!CanRegister())
            {
                RegisterSuccessMessage("This wiki only allows private registration.  Please contact an administrator.");

                return WikiIndex();
            }

            return View(Models.RegisterModel());
        }

        private bool CanRegister()
        {
            if (UserSession.IsAdministrator)
            {
                return true;
            }

            using(DatabaseConnectionFactory.Create(WikiData.Source))
            {
                return repository.Count == 0 || !SystemSettings().PrivateRegistration;
            }
        }

        [HttpPost]
        public ActionResult Add()
        {
            if (!CanRegister())
            {
                return WikiIndex();
            }

            var model = Models.RegisterModel();
            var form = Request.Form;

            model.MemberName = MemberView.MemberName.MapFrom(form);
            model.EMailAddress = MemberView.EMailAddress.MapFrom(form);

            var password = PasswordView.Password.MapFrom(form);

            int count;
            bool first;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                if (repository.ContainsName(model.MemberName))
                {
                    RegisterFailureMessage(string.Format("There is already a member with name '{0}'.  Please use a different name.", model.MemberName));

                    return View(Actions.Register, model);
                }

                var member = new Member
                                 {
                                     Name = model.MemberName,
                                     EMailAddress = new EMailAddress(model.EMailAddress),
                                     NewEMailAddress = new EMailAddress(model.EMailAddress),
                                     RegistrationDate = DateTime.Now,
                                     Role = MemberRole.Member.ToString()
                                 };

                member.SetPasswordPlain(password, CryptographyService, ApplicationConfiguration);

                count = repository.Count;
                first = count == 0;

                if (first)
                {
                    member.MakeAdministrator();
                    member.Activate();
                }

                repository.Add(member);

                if (!first)
                {
                    memberActivationTask.SendActivationEMail(member);
                }
                else
                {
                    memberLoginTask.LogIn(model.MemberName, password);
                }
            }

            RegisterSuccessMessage(string.Format("You have succesfully registered with name '{0}'.", model.MemberName));

            RegisterSuccessMessage(
                !first
                    ? "An e-mail has been sent to the address you registered with.  Please click on the activation link in the e-mail to activate your account."
                    : "Since you are the first member to register you have been made the administrator and your account has been activated.");

            return WikiIndex();
        }

        [HttpGet]
        [MemberAccess]
        public ActionResult Profile()
        {
            DataRow row;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                row = query.Details(UserSession.MemberId);
            }

            var activationDate = MemberColumns.ActivationDate.MapFrom(row);

            return View(new MemberProfileModel
                            {
                                Name = MemberColumns.Name.MapFrom(row),
                                EMailAddress = MemberColumns.EMailAddress.MapFrom(row),
                                RegistrationDate = MemberColumns.RegistrationDate.MapFrom(row).FormatFull(),
                                ActivationDate = activationDate.HasValue ? activationDate.Value.FormatFull() : "(not yet activated)"
                            });
        }

        [HttpPost]
        [MemberAccess(MemberRole.Administrator)]
        public ActionResult Delete()
        {
            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                var member = repository.Get(RouteData.IdAsGuid());

                if (member == null)
                {
                    return JsonError("Member id does not exist.");
                }

                repository.Remove(member);
            }
            
            return JsonSuccess();
        }

        [HttpGet]
        [MemberAccess]
        public ActionResult NewPassword()
        {
            return View(new FormModel(Actions.ChangePassword, Name)
                            {
                                SubmitButtonText = "Change Password"
                            });
        }

        [HttpPost]
        [MemberAccess]
        public ActionResult ChangePassword()
        {
            Bus.Send(new ChangePasswordCommand
                        {
                            MemberId = UserSession.MemberId,
                            Password = PasswordView.Password.MapFrom(Request.Form)
                        });

            RegisterSuccessMessage("Your password has been changed.");

            return RedirectToAction(WikiPageController.Actions.Index, WikiPageController.Name);
        }

        [HttpGet]
        public ActionResult Contact()
        {
            List<KeyValuePair<string, string>> list;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                list = (from DataRow row in query.All().Rows
                        select
                            new KeyValuePair<string, string>(MemberColumns.Id.MapFrom(row).ToString(), MemberColumns.Name.MapFrom(row)))
                    .ToList();
            }

            return View(new ContactModel(new SelectList(list, "Key", "Value")));
        }

        [HttpPost]
        public ActionResult SendMail()
        {
            var form = Request.Form;
            DataRow memberRow;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                memberRow = query.Details(new Guid(ContactView.MemberName.MapFrom(form)));
            }

            EMailGateway.SendMail(new MailMessage(ContactView.YourEMailAddress.MapFrom(form),
                                                  MemberColumns.EMailAddress.MapFrom(memberRow),
                                                  "Message from Shuttle ESB wiki site", ContactView.Message.MapFrom(form)));

            RegisterSuccessMessage(string.Format("Your message has been sent to {0}.", MemberColumns.Name.MapFrom(memberRow)));

            return WikiIndex();
        }

        [HttpGet]
        public ActionResult Activate()
        {
            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                if (!RouteData.Id().IsGuid())
                {
                    RegisterFailureMessage("Invalid member id.");
                }
                else
                {
                    switch (memberActivationTask.Activate(RouteData.IdAsGuid()))
                    {
                        case ActivationResult.MemberNotFound:
                            {
                                RegisterFailureMessage("Member not found.");
                                return WikiIndex();
                            }
                        case ActivationResult.AlreadyActivated:
                            {
                                RegisterFailureMessage("Your account has already been activated.");
                                break;
                            }
                        case ActivationResult.Activated:
                            {
                                RegisterFailureMessage("Your account has been activated.");
                                break;
                            }
                    }
                }
            }

            return RedirectToAction(Actions.Credentials);
        }

        [HttpGet]
        public ActionResult ActivateEMail()
        {
            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                if (!RouteData.Id().IsGuid())
                {
                    RegisterFailureMessage("Invalid member id.");
                }
                else
                {
                    switch (memberActivationTask.ActivateEMail(RouteData.IdAsGuid()))
                    {
                        case ActivationResult.MemberNotFound:
                            {
                                RegisterFailureMessage("Member not found.");
                                return WikiIndex();
                            }
                        case ActivationResult.AlreadyActivated:
                            {
                                RegisterFailureMessage("Your new e-mail address has already been activated.");
                                break;
                            }
                        case ActivationResult.Activated:
                            {
                                RegisterFailureMessage("Your new e-mail address has been activated.");
                                break;
                            }
                    }
                }
            }

            return RedirectToAction(Actions.Credentials);
        }

        [HttpGet]
        [MemberAccess(MemberRole.Administrator)]
        public ActionResult Manage()
        {
            DataTable table;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                table = query.All();
            }

            return View(table);
        }

        public static class Actions
        {
            public static readonly string Add = "Add";
            public static readonly string ApplyNewEMailAddress = "ApplyNewEMailAddress";
            public static readonly string ChangePassword = "ChangePassword";
            public static readonly string Contact = "Contact";
            public static readonly string Credentials = "Credentials";
            public static readonly string CredentialsReminder = "CredentialsReminder";
            public static readonly string Login = "Login";
            public static readonly string Logout = "Logout";
            public static readonly string Manage = "Manage";
            public static readonly string NewPassword = "NewPassword";
            public static readonly string Profile = "Profile";
            public static readonly string Register = "Register";
            public static readonly string SendCredentials = "SendCredentials";
            public static readonly string SendMail = "SendMail";
        }

        private static class Models
        {
            public static RegisterModel RegisterModel()
            {
                return new RegisterModel(Actions.Add, Name)
                {
                    SubmitButtonText = "Register"
                };
            }

            public static EMailModel EMailModel(string emailAddress)
            {
                return new EMailModel(Actions.ApplyNewEMailAddress, Name)
                {
                    EMailAddress = emailAddress
                };
            }

        }
    }
}