﻿using System;
using System.Data;
using System.Web.Mvc;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki.MVC
{
	public class WikiPageController : ShuttleController
	{
		public static string Name = "WikiPage";

		private readonly IWikiRepository wikiRepository;
		private readonly IWikiPageQuery wikiPageQuery;

		public WikiPageController(IWikiRepository wikiRepository, IWikiPageQuery wikiPageQuery)
		{
			this.wikiRepository = wikiRepository;
			this.wikiPageQuery = wikiPageQuery;
		}

		protected override void HandleUnknownAction(string actionName)
		{
			if (Request.HttpMethod.Equals("GET", StringComparison.CurrentCultureIgnoreCase))
			{
				DataTable result;

				using (DatabaseConnectionFactory.Create(WikiData.Source))
				{
					result = wikiPageQuery.GetPageHtml(actionName);
				}

				if (result.Row() != null)
				{
					var html = WikiPageColumns.Html.MapFrom(result.Row());

					if (string.IsNullOrEmpty(html))
					{
						using (DatabaseConnectionFactory.Create(WikiData.Source))
						{
							html = wikiRepository.RenderAndSave(WikiPageColumns.Id.MapFrom(result.Row()));
						}
					}

					ViewData.Model = new DisplayPageModel(actionName, html);
					View(Views.DisplayPage).ExecuteResult(ControllerContext);

                    UserSession.AddRecentPageName(actionName);

					return;
				}

				ViewData.Model = new UnknownPageModel(actionName, UserSession);
				View(Views.UnknownPage).ExecuteResult(ControllerContext);

				return;
			}

			base.HandleUnknownAction(actionName);
		}

		[HttpGet]
		[MemberAccess]
		public ActionResult ChangePageName()
		{
			return View(new ChangePageNameModel(Actions.RenamePage, Name)
							{
								PageName = RouteData.Id()
							});
		}

		[HttpGet]
		[MemberAccess]
		public ActionResult NewPage()
		{
			try
			{
				return View(new PageModel(Actions.CreatePage, Name)
								{
									PageName = RouteData.Id()
								});
			}
			catch (Exception ex)
			{
				Log.Error(ex.CompactMessages());
			}

			return null;
		}

		[HttpPost]
		[MemberAccess]
		[ValidateInput(false)]
		public ActionResult CreatePage()
		{
			var pageName = WikiPageView.PageName.MapFrom(Request.Form);

			if (string.IsNullOrEmpty(pageName))
			{
				RegisterFailureMessage("The page name is not available.  If you entered 'newpage' in the url (address bar) is may cause this behaviour.");

				return WikiIndex();
			}

			Bus.Send(new CreateWikiPageCommand
						{
							PageName = pageName,
							Markup = SectionView.WikiMarkup.MapFrom(Request.Form)
						});

			return RedirectToAction(pageName, Name);
		}

		[HttpGet]
		[MemberAccess]
		public ActionResult Edit()
		{
			var sectionModel = new SectionModel(Actions.SaveSection, Name);

			WikiSection section;

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				sectionModel.Id = RouteData.IdAsGuid();
				section = wikiRepository.GetSection(RouteData.IdAsGuid());

				if (section == null)
				{
					RegisterFailureMessage("The requested section no longer exists.");

					return WikiIndex();
				}

				sectionModel.Markup = section.GetMarkup();
			}

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				sectionModel.PageName = WikiPageColumns.PageName.MapFrom(wikiPageQuery.GetPageNameFromSectionId(section.Id).Row());
			}

			return View(sectionModel);
		}

		[HttpPost]
		[MemberAccess]
		[ValidateInput(false)]
		public ActionResult SaveSection()
		{
			var pageName = WikiPageView.PageName.MapFrom(Request.Form);

			Bus.Send(new SaveWikiSectionCommand
						{
							PageName = pageName,
							SectionId = RouteData.IdAsGuid(),
							Markup = SectionView.WikiMarkup.MapFrom(Request.Form)
						});

			return RedirectToAction(pageName, Name);
		}

		[HttpPost]
		[MemberAccess]
		public ActionResult RenamePage()
		{
			var pageName = WikiPageView.NewPageName.MapFrom(Request.Form);

			Bus.Send(new RenameWikiPageCommand
						{
							NewPageName = WikiPageView.NewPageName.MapFrom(Request.Form),
							PageName = WikiPageView.PageName.MapFrom(Request.Form)
						});

			return RedirectToAction(pageName, Name);
		}

		[HttpGet]
		[MemberAccess]
		public ActionResult DeleteSection()
		{
			var sectionId = RouteData.IdAsGuid();
			string pageName;
			
			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				pageName = WikiPageColumns.PageName.MapFrom(wikiPageQuery.GetPageNameFromSectionId(sectionId).Row());
			}

			Bus.Send(new DeleteWikiSectionCommand
						{
							PageName = pageName,
							SectionId = sectionId
						});

			return RedirectToAction(pageName, Name);
		}

		[HttpGet]
		[MemberAccess]
		public ActionResult DeletePage()
		{
			var pageName = RouteData.Id();

			Bus.Send(new DeleteWikiPageCommand
						{
							PageName = pageName
						});

			RegisterSuccessMessage(string.Format("Page '{0}' has been deleted.", pageName));

			return WikiIndex();
		}

		[HttpGet]
		public ActionResult NoJS()
		{
			return View();
		}

		public static class Views
		{
			public static readonly string DisplayPage = "DisplayPage";
			public static readonly string UnknownPage = "UnknownPage";
		}

		public static class Actions
		{
			public static readonly string ChangePageName = "ChangePageName";
			public static readonly string CreatePage = "CreatePage";
			public static readonly string DeleteSection = "DeleteSection";
			public static readonly string DeletePage = "DeletePage";
            public static readonly string MarkupGuide = "MarkupGuide";
			public static readonly string RenamePage = "RenamePage";
			public static readonly string SaveSection = "SaveSection";
			public static readonly string Index = "Index";
		}
	}
}