﻿using System;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using log4net;
using log4net.Config;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;
using Shuttle.Core.Infrastructure.Log4Net;

namespace Shuttle.Wiki.MVC
{
    public class MvcApplication : HttpApplication
    {
		private static Exception startupException;
		
		public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");
            routes.IgnoreRoute("{resource}.ico");
            routes.IgnoreRoute("content/{*anything}");
            routes.IgnoreRoute("js/{*anything}");
            routes.IgnoreRoute("{resource}.ashx");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new {controller = "WikiPage", action = "Index", id = UrlParameter.Optional} // Parameter defaults
                );
        }

        protected void Application_Start()
        {
        	try
        	{
        		XmlConfigurator.Configure();

        		Log.Assign(new Log4NetLog(LogManager.GetLogger(typeof(MvcApplication))));

        		Log.Information("Shuttle.Wiki Started.");

                ConnectionStrings.Approve();

        		ControllerBuilder.Current.SetControllerFactory(new ShuttleControllerFactory());

        		AreaRegistration.RegisterAllAreas();

        		RegisterGlobalFilters(GlobalFilters.Filters);
        		RegisterRoutes(RouteTable.Routes);

        		DependencyWiring
        			.Start()
        			.AddServerComponents()
        			.AddWebComponents();

                ViewEngines.Engines.Clear();

        		ViewEngines.Engines.Add(new RazorCSharpViewEngine());
        	}
        	catch (Exception ex)
        	{
				startupException = ex;
			}
        }

		protected void Application_BeginRequest(object sender, EventArgs e)
		{
			if (startupException == null)
			{
				return;
			}

			var html = new StringBuilder();

			html.Append("<html><head><title>Shuttle Wiki Application Startup Exception</title><style>");

			html.Append(
				"body { background: none repeat scroll 0 0 #CBE1EF; font-family: 'MS Tresbuchet',Verdana,Arial,Helvetica,sans-serif; font-size: 0.7em; margin: 0; }");
			html.Append(
				"div.header { background-color: #5c87b2; color: #ffffff; padding: .2em 2%; font-size: 2em; font-weight: bold; }");
			html.Append(
				"div.error { border: solid 1px #5a7fa5; background-color: #ffffff; color: #CC0000; padding: .5em; font-size: 2em; width: 96%; margin: .5em auto; }");
			html.Append(
				"div.information { border: solid 1px #5a7fa5; background-color: #ffffff; color: #555555; padding: 1em; font-size: 1em; width: 96%; margin: 1em auto; }");

			html.Append("</style></head><body>");
			html.Append("<div class='header'>Shuttle Wiki Application Startup Exception</div>");
			html.AppendFormat("<div class='error'><b>source</b>:<br>{0}</div>", startupException.Source);
			html.AppendFormat("<div class='information'><b>message</b>:<br>{0}</div>", startupException.CompactMessages());

			var crlf = new Regex(@"(\r\n|\r|\n)+");

			html.AppendFormat("<div class='information'><b>stack trace</b>:<br>{0}</div>", crlf.Replace(startupException.StackTrace, "<br/>"));

			var reflection = startupException as ReflectionTypeLoadException;

			if (reflection != null)
			{
				html.Append("<div class='information'><b>loader exception(s)</b>:<br>");

				foreach (var exception in reflection.LoaderExceptions)
				{
					html.AppendFormat(string.Format("{0}<br/>", exception.CompactMessages()));
				}

				html.Append("</div>");
			}

			html.Append("</body></html>");

			HttpContext.Current.Response.Write(html);
			HttpContext.Current.Response.End();
		}

		protected void Application_End()
		{
			Log.Information("Shuttle.Wiki Stopped.");

			LogManager.Shutdown();
		}

		public void Application_Error(object sender, EventArgs e)
		{
			Log.Information(Server.GetLastError().CompactMessages());

			var exception = Server.GetLastError() as ReflectionTypeLoadException;

			if (exception != null)
			{
				exception.LoaderExceptions.ForEach(ex => Log.Information(ex.ToString()));
			}
		}
	}
}