using System.Linq;
using NUnit.Framework;

namespace Shuttle.Wiki.Test
{
    public class WikiPageTest : Fixture
    {
        [Test]
        public void Should_be_able_to_add_wiki_sections()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("! Heading 1", "section1");
            var section2 = new WikiSection("! Heading 2", "section2");
            var section3 = new WikiSection("! Heading 3", "section3");

            page.Add(section1);

            var html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsFalse(html.Contains(section2.BodyMarkup));
            Assert.IsFalse(html.Contains(section3.BodyMarkup));

            page.Add(section2);

            html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsTrue(html.Contains(section2.BodyMarkup));
            Assert.IsFalse(html.Contains(section3.BodyMarkup));

            page.Add(section3);

            html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsTrue(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));
        }

        [Test]
        public void Should_not_add__empty_wiki_sections()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("! Heading 1", "section1");
            var section2 = new WikiSection(string.Empty, string.Empty);
            var section3 = new WikiSection("! Heading 3", "section3");

            page.Add(section1);
            page.Add(section2);
            page.Add(section3);

            var html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsFalse(page.Contains(section2));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));
        }

        [Test]
        public void Should_be_able_to_remove_wiki_sections()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("! Heading 1", "section1");
            var section2 = new WikiSection("! Heading 2", "section2");
            var section3 = new WikiSection("! Heading 3", "section3");

            page.Add(section1);
            page.Add(section2);
            page.Add(section3);

            var html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsTrue(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));

            page.Remove(section2);

            html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsFalse(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));
        }

        [Test]
        public void Should_be_able_to_replace_wiki_sections()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("! Heading 1", "section1");
            var section2 = new WikiSection("! Heading 2", "section2");
            var section3 = new WikiSection("! Heading 3", "section3");
            var section4 = new WikiSection("! Heading 4", "section4");
            var section5 = new WikiSection("! Heading 5", "section5");

            page.Add(section1);
            page.Add(section2);
            page.Add(section3);

            var html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsTrue(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));

            page.Replace(section2, new[] { section4, section5 });

            html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsFalse(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));
            Assert.IsTrue(html.Contains(section4.BodyMarkup));
            Assert.IsTrue(html.Contains(section5.BodyMarkup));
        }

        [Test]
        public void Should_remove_section_if_replaced_by_empty_wiki_section()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("! Heading 1", "section1");
            var section2 = new WikiSection("! Heading 2", "section2");
            var section3 = new WikiSection("! Heading 3", "section3");
            var sectionEmpty = new WikiSection(string.Empty, string.Empty);

            page.Add(section1);
            page.Add(section2);
            page.Add(section3);

            var html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsTrue(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));

            page.Replace(section2, new[] { sectionEmpty });

            html = page.Render();

            Assert.IsTrue(html.Contains(section1.BodyMarkup));
            Assert.IsFalse(html.Contains(section2.BodyMarkup));
            Assert.IsTrue(html.Contains(section3.BodyMarkup));
        }

        [Test]
        public void Should_be_able_to_normalize_markup()
        {
            var page = new WikiPage("page");

            var section1 = new WikiSection("", "section1");
            var section2 = new WikiSection("! Heading 2", "section2");
            var section3 = new WikiSection("", "section3");

            page.Add(section1);
            page.Add(section2);
            page.Add(section3);

            Assert.AreEqual(3, page.Sections.Count());

            page.NormalizeSections();

            Assert.AreEqual(2, page.Sections.Count());
        }
    }
}