﻿using System;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class WikiRepository : Repository, IWikiRepository
    {
        private readonly IAssembler<WikiPage> assembler;
        private readonly IDataRepository<WikiPage> pageDataRepository;
        private readonly IDataRepository<WikiSection> sectionDataRepository;

        public WikiRepository(IAssembler<WikiPage> assembler,
                              IDataRepository<WikiPage> pageDataRepository,
                              IDataRepository<WikiSection> sectionDataRepository)
        {
            Guard.AgainstNull(assembler, "assembler");
            Guard.AgainstNull(pageDataRepository, "pageDataRepository");
            Guard.AgainstNull(sectionDataRepository, "sectionDataRepository");

            this.assembler = assembler;
            this.pageDataRepository = pageDataRepository;
            this.sectionDataRepository = sectionDataRepository;
        }

        public WikiPage GetPageByName(string pageName)
        {
            return Assemble(pageDataRepository.FetchMappedRowUsing(WikiData.Source, WikiPageTableAccess.GetByName(pageName)));
        }

        public bool ContainsPageName(string pageName)
        {
            return DatabaseGateway.GetScalarUsing<int>(WikiData.Source, WikiPageTableAccess.ContainsName(pageName)) == 1;
        }

        public WikiSection GetSection(Guid sectionId)
        {
            return sectionDataRepository.FetchItemUsing(WikiData.Source, WikiSectionTableAccess.Get(sectionId));
        }

        public void Save(WikiPage wikiPage, string historicalMarkup, IUserSession userSession)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageHistoryTableAccess.Add(wikiPage, historicalMarkup, userSession));

            Save(wikiPage);
        }

    	public void Remove(WikiPage wikiPage, string historicalMarkup, IUserSession userSession)
    	{
			DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageHistoryTableAccess.Add(wikiPage, historicalMarkup, userSession));

			Remove(wikiPage);
		}

    	public string RenderAndSave(Guid pageId)
    	{
    		var page = Get(pageId);

			if (page == null)
			{
				return string.Empty;
			}

    		var html = page.Render();

    		DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageTableAccess.SaveRendering(page.Id, html));

    		return html;
    	}

    	public void Rename(string newPageName, string pageName)
    	{
			DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageTableAccess.Rename(newPageName, pageName));
			DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageHistoryTableAccess.Rename(newPageName, pageName));
    	}

    	public WikiPage Get(Guid id)
        {
            return Assemble(pageDataRepository.FetchMappedRowUsing(WikiData.Source, WikiPageTableAccess.Get(id)));
        }

        private WikiPage Assemble(MappedRow<WikiPage> pageMappedRow)
        {
            if (pageMappedRow == null)
            {
                return null;
            }

            return
                assembler.AssembleItem(
                    new MappedData()
                        .Add(pageMappedRow)
                        .Add(
                            sectionDataRepository.FetchMappedRowsUsing(
                                WikiData.Source,
                                WikiSectionTableAccess.GetSections(WikiPageColumns.Id.MapFrom(pageMappedRow.Row)))));
        }

        public void Add(WikiPage entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageTableAccess.Add(entity));

            var sequence = 1;

            foreach (var section in entity.Sections)
            {
                DatabaseGateway.ExecuteUsing(WikiData.Source, WikiSectionTableAccess.Add(entity, section, sequence++));
            }
        }

        public void Save(WikiPage entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiSectionTableAccess.Remove(entity));

            var sequence = 1;

            foreach (var section in entity.Sections)
            {
                DatabaseGateway.ExecuteUsing(WikiData.Source, WikiSectionTableAccess.Add(entity, section, sequence++));
            }

            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageTableAccess.SaveRendering(entity.Id, entity.Render()));
        }

        public void Remove(WikiPage entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiPageTableAccess.Remove(entity));
        }


    }
}