using Castle.MicroKernel.Registration;
using Castle.Windsor;
using Shuttle.Core.Data;
using Shuttle.Core.Data.Castle;
using Shuttle.Core.Domain;
using Shuttle.Core.Infrastructure;
using Shuttle.Core.Infrastructure.Castle;
using Shuttle.Core.Web;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    public interface IDependencyWiring
    {
        IDependencyWiring AddServerComponents();
        IDependencyWiring AddWebComponents();
    }

    public class DependencyWiring : IDependencyWiring
    {
        private readonly WindsorContainer container = new WindsorContainer();

        private DependencyWiring()
        {
        }

        public IDependencyWiring AddServerComponents()
        {
            container.RegisterDataAccess("Shuttle.Wiki");
            container.RegisterSingleton("Shuttle.Wiki", RegexPatterns.EndsWith("Task"));
            container.RegisterTransient("Shuttle.Wiki", typeof(IMessageHandler<>));

            return this;
        }

        public IDependencyWiring AddWebComponents()
        {
            container.Register(
                AllTypes
                    .FromAssemblyNamed("Shuttle.Wiki.MVC")
                    .Pick()
                    .If(type => !type.IsInterface && type.Name.EndsWith("Controller"))
                    .Configure(configurer => configurer.Named(configurer.Implementation.Name.ToLower()))
                    .Configure(configurer => configurer.LifeStyle.Transient)
                    .WithService.FirstInterface());

            container.Register(Component.For<IUserSession>().ImplementedBy<UserSession>());
            container.Register(Component.For<IServiceBus>().ImplementedBy<MemoryBus>());

            MemoryBus.InitializeWith(Resolver.Instance);

            return this;
        }

        public static IDependencyWiring Start()
        {
            return new DependencyWiring().AddCore();
        }

        private IDependencyWiring AddCore()
        {
            // Infrastructure
            container.Register(Component.For<IApplicationConfiguration>().ImplementedBy<ApplicationConfiguration>());
            container.Register(Component.For<IFileService>().ImplementedBy<FileService>());
            container.Register(Component.For<IObjectSerializer>().ImplementedBy<XmlObjectSerializer>());
            container.Register(Component.For<IImageService>().ImplementedBy<ImageService>());
            container.Register(Component.For<ICryptographyService>().ImplementedBy<CryptographyService>());
            container.Register(Component.For<IEMailGateway>().ImplementedBy<EMailGateway>());

            // Data Access
            container.RegisterDataAccessCore();
            container.RegisterDataAccessDefaults();

            container.RegisterSingleton<IRepositoryProvider, FaultRepositoryProvider>();
            container.RegisterSingleton<IDatabaseConnectionCache, HttpDatabaseConnectionCache>();

            Resolver.Assign(new WindsorResolver(container));

            return this;
        }
    }
}