using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Shuttle.Core.Domain;
using Shuttle.Core.Infrastructure;
using WikiPlex;
using WikiPlex.Compilation.Macros;
using WikiPlex.Formatting.Renderers;

namespace Shuttle.Wiki
{
    public class WikiPage : RowIdentity
    {
        private const int MAX_HEADING_MARKUP_LENGTH = 250;
        private const int MAX_BODY_MARKUP_LENGTH = 7000;

        private HumaneList<WikiSection> sections = new HumaneList<WikiSection>();

        public WikiPage(string name)
        {
            Name = name;
        }

        public WikiPage(Guid id, string name)
            : base(id)
        {
            Name = name;
        }

        public string Name { get; private set; }

        public IEnumerable<WikiSection> Sections
        {
            get { return sections.AsReadOnly(); }
        }

        public void Add(WikiSection section)
        {
            Guard.AgainstNull(section, "section");

            if (Contains(section))
            {
                throw new DuplicateEntryException(string.Format(InfrastructureResources.DuplicateEntryException,
                                                                section.Id, "sections"));
            }

            if (section.GetMarkup().Length == 0)
            {
                return;
            }

            sections.Add(section);
        }

        public bool Contains(WikiSection section)
        {
            Guard.AgainstNull(section, "section");

            return Contains(section.Id);
        }

        public bool Contains(Guid id)
        {
            return sections.Find(section => section.Id.Equals(id)) != null;
        }

        public string Render()
        {
            var engine = new WikiEngine();

            var macros = Macros.All.Union(new IMacro[]
                                          {
                                          	new WikiPageLinkMacro(),
											new WikiImageMacro()
                                          });
            var renderers = Renderers.All.Union(new IRenderer[]
                                                {
                                                	new WikiPageLinkRenderer(),
													new WikiImageRenderer()
                                                });

            var html = new StringBuilder();

            foreach (var section in sections)
            {
                html.AppendFormat(
                    "<div class='wiki-section ui-helper-clearfix'><div class='wiki-heading ui-helper-clearfix'>{0}<a href='/wikipage/edit/{1}' class='edit-wiki-section'>Edit</a></div>",
                    engine.Render(section.HeadingMarkup, macros, renderers),
                    section.Id);
                html.AppendLine();
                html.AppendLine(engine.Render(section.BodyMarkup, macros, renderers));
				html.AppendLine("</div>");
			}


            return html.ToString();
        }

        public void Remove(WikiSection section)
        {
            Guard.AgainstNull(section, "section");

            Remove(section.Id);
        }

        public void Remove(Guid id)
        {
            sections.RemoveAll(item => item.Id.Equals(id));
        }

        public void Replace(WikiSection section, IEnumerable<WikiSection> withSections)
        {
            Guard.AgainstNull(section, "section");

            Replace(section.Id, withSections);
        }

        public void Replace(Guid id, IEnumerable<WikiSection> withSections)
        {
            var index = sections.FindIndex(item => item.Id.Equals(id));

            Remove(id);

            if (withSections == null || withSections.Count() == 0)
            {
                return;
            }

            foreach (var section in withSections)
            {
                sections.Insert(index, section);

                index++;
            }
        }

        public void EnforceInvariants()
        {
            if (sections.Any(section => section.HeadingMarkup.Length > MAX_HEADING_MARKUP_LENGTH))
            {
                throw new InvariantException(string.Format(Resources.BodyMarkupTooLong, Name, MAX_BODY_MARKUP_LENGTH));
            }

            if (sections.Any(section => section.BodyMarkup.Length > MAX_BODY_MARKUP_LENGTH))
            {
                throw new InvariantException(string.Format(Resources.BodyMarkupTooLong, Name, MAX_BODY_MARKUP_LENGTH));
            }
        }

        public void NormalizeSections()
        {
            sections = new HumaneList<WikiSection>(new WikiMarkup(GetMarkup()).GetSections());
        }

        public string GetMarkup()
        {
            var markup = new StringBuilder();

            foreach (var section in sections)
            {
                markup.AppendLine(section.HeadingMarkup);
                markup.AppendLine(section.BodyMarkup);
            }

            return markup.ToString();
        }
    }
}