using System;
using System.Collections.Generic;
using WikiPlex.Common;
using WikiPlex.Formatting.Renderers;

namespace Shuttle.Wiki
{
	internal enum FloatAlignment
	{
		None,
		Left,
		Right
	}

	internal static class FloatAlignmentExtensions
	{
		internal static string GetStyle(this FloatAlignment alignment)
		{
			switch (alignment)
			{
				case FloatAlignment.Left:
					return "left";
				case FloatAlignment.Right:
					return "right";
				default:
					return string.Empty;
			}
		}

		internal static string GetPadding(this FloatAlignment alignment)
		{
			switch (alignment)
			{
				case FloatAlignment.Left:
					return "padding-right:.5em;";
				case FloatAlignment.Right:
					return "padding-left:.5em;";
				default:
					return string.Empty;
			}
		}
	}

	public class WikiImageRenderer : Renderer
	{
		private const string WrapperFormat = "<div class='wiki-image alignment-{0}'>{1}</div>";
		private const string ImageAndLink = "<a href=\"{2}\"><img style=\"border:none;\" src=\"{3}\" {4}/></a>";
		private const string ImageAndLinkWithStyle = "<div style=\"clear:both;height:0;\">&nbsp;</div><a style=\"float:{0};{1}\" href=\"{2}\"><img style=\"border:none;\" src=\"{3}\" {4}/></a>";
		private const string ImageLinkAndAlt = "<a href=\"{2}\"><img style=\"border:none;\" src=\"{3}\" alt=\"{4}\" title=\"{4}\" {5}/></a>";
		private const string ImageLinkAndAltWithStyle = "<div style=\"clear:both;height:0;\">&nbsp;</div><a style=\"float:{0};{1}\" href=\"{2}\"><img style=\"border:none;\" src=\"{3}\" alt=\"{4}\" title=\"{4}\" {5}/></a>";
		private const string ImageNoLink = "<img src=\"{2}\" {3}/>";
		private const string ImageNoLinkAndAlt = "<img src=\"{2}\" alt=\"{3}\" title=\"{3}\" {4}/>";
		private const string ImageNoLinkAndAltWithStyle = "<div style=\"clear:both;height:0;\">&nbsp;</div><img style=\"float:{0};{1}\" src=\"{2}\" alt=\"{3}\" title=\"{3}\" {4}/>";
		private const string ImageNoLinkWithStyle = "<div style=\"clear:both;height:0;\">&nbsp;</div><img style=\"float:{0};{1}\" src=\"{2}\" {3}/>";

		protected override ICollection<string> ScopeNames
		{
			get
			{
				return new[]
				       {
				       	WikiScopeName.ImageWithLinkNoAltLeftAlign, WikiScopeName.ImageWithLinkNoAltRightAlign,
				       	WikiScopeName.ImageWithLinkNoAlt, WikiScopeName.ImageWithLinkWithAltLeftAlign,
				       	WikiScopeName.ImageWithLinkWithAltRightAlign, WikiScopeName.ImageWithLinkWithAlt,
				       	WikiScopeName.ImageLeftAlign, WikiScopeName.ImageRightAlign,
				       	WikiScopeName.ImageNoAlign, WikiScopeName.ImageLeftAlignWithAlt,
				       	WikiScopeName.ImageRightAlignWithAlt, WikiScopeName.ImageNoAlignWithAlt
				       };
			}
		}

		/// <summary>
		/// Gets the invalid macro error text.
		/// </summary>
		protected override string InvalidMacroError
		{
			get
			{
				return "Cannot resolve wiki image macro, invalid number of parameters.";
			}
		}

		/// <summary>
		/// Will expand the input into the appropriate content based on scope.
		/// </summary>
		/// <param name="scopeName">The scope name.</param>
		/// <param name="input">The input to be expanded.</param>
		/// <param name="htmlEncode">Function that will html encode the output.</param>
		/// <param name="attributeEncode">Function that will html attribute encode the output.</param>
		/// <returns>The expanded content.</returns>
		protected override string PerformExpand(string scopeName, string input, Func<string, string> htmlEncode, Func<string, string> attributeEncode)
		{
			var alignment = GetAlignment(scopeName);
			var renderMethod = GetRenderMethod(scopeName);

			try
			{
				return renderMethod(input, alignment, attributeEncode);
			}
			catch
			{
				throw new RenderException();
			}
		}

		private static FloatAlignment GetAlignment(string scopeName)
		{
			switch (scopeName)
			{
				case WikiScopeName.ImageLeftAlign:
				case WikiScopeName.ImageLeftAlignWithAlt:
				case WikiScopeName.ImageWithLinkNoAltLeftAlign:
				case WikiScopeName.ImageWithLinkWithAltLeftAlign:
					return FloatAlignment.Left;
				case WikiScopeName.ImageRightAlign:
				case WikiScopeName.ImageRightAlignWithAlt:
				case WikiScopeName.ImageWithLinkNoAltRightAlign:
				case WikiScopeName.ImageWithLinkWithAltRightAlign:
					return FloatAlignment.Right;
				default:
					return FloatAlignment.None;
			}
		}

		private static Func<string, FloatAlignment, Func<string, string>, string> GetRenderMethod(string scopeName)
		{
			switch (scopeName)
			{
				case WikiScopeName.ImageLeftAlign:
				case WikiScopeName.ImageRightAlign:
				case WikiScopeName.ImageNoAlign:
					return RenderImageNoLinkMacro;
				case WikiScopeName.ImageLeftAlignWithAlt:
				case WikiScopeName.ImageRightAlignWithAlt:
				case WikiScopeName.ImageNoAlignWithAlt:
					return RenderImageWithAltMacro;
				case WikiScopeName.ImageWithLinkNoAlt:
				case WikiScopeName.ImageWithLinkNoAltLeftAlign:
				case WikiScopeName.ImageWithLinkNoAltRightAlign:
					return RenderImageWithLinkMacro;
				case WikiScopeName.ImageWithLinkWithAlt:
				case WikiScopeName.ImageWithLinkWithAltLeftAlign:
				case WikiScopeName.ImageWithLinkWithAltRightAlign:
					return RenderImageWithLinkAndAltMacro;
			}

			return null;
		}

		private static string RenderImageNoLinkMacro(string input, FloatAlignment alignment, Func<string, string> encode)
		{
			string format = alignment == FloatAlignment.None ? ImageNoLink : ImageNoLinkWithStyle;
			ImagePart parts = Utility.ExtractImageParts(input, ImagePartExtras.None);

			return string.Format(format, alignment.GetStyle(), alignment.GetPadding(), encode(ImageUrl(parts)), parts.Dimensions);
		}

		private static string ImageUrl(ImagePart parts)
		{
			return string.Format("/image/raw/{0}", parts.ImageUrl);
		}

		private static string RenderImageWithAltMacro(string input, FloatAlignment alignment, Func<string, string> encode)
		{
			string format = alignment == FloatAlignment.None ? ImageNoLinkAndAlt : ImageNoLinkAndAltWithStyle;
			ImagePart parts = Utility.ExtractImageParts(input, ImagePartExtras.ContainsText);

			return string.Format(format, alignment.GetStyle(), alignment.GetPadding(), encode(ImageUrl(parts)), encode(parts.Text), parts.Dimensions);
		}

		private static string RenderImageWithLinkMacro(string input, FloatAlignment alignment, Func<string, string> encode)
		{
			string format = alignment == FloatAlignment.None ? ImageAndLink : ImageAndLinkWithStyle;
			ImagePart parts = Utility.ExtractImageParts(input, ImagePartExtras.ContainsLink);

			return string.Format(format, alignment.GetStyle(), alignment.GetPadding(), encode(parts.LinkUrl), encode(ImageUrl(parts)), parts.Dimensions);
		}

		private static string RenderImageWithLinkAndAltMacro(string input, FloatAlignment alignment, Func<string, string> encode)
		{
			string format = alignment == FloatAlignment.None ? ImageLinkAndAlt : ImageLinkAndAltWithStyle;
			ImagePart parts = Utility.ExtractImageParts(input, ImagePartExtras.All);

			return string.Format(format, alignment.GetStyle(), alignment.GetPadding(), encode(parts.LinkUrl), encode(ImageUrl(parts)), encode(parts.Text), parts.Dimensions);
		}
	}
}