using System;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using System.Web.Routing;
using Shuttle.Core.Infrastructure;
using Shuttle.Core.Web;

namespace Shuttle.Wiki
{
    public static class HtmlHelperExtensions
    {
#if !DEBUG
        private const string VERSION = "20111021";
#endif

        private static string Version()
        {
#if DEBUG
            return Guid.NewGuid().ToString();
#else
            return VERSION;
#endif
        }

        public static HtmlString Button(this HtmlHelper html, string name, string text)
        {
            return Button(html, name, text, null);
        }

        public static HtmlString Button(this HtmlHelper html, string name, string text, object htmlAttributes)
        {
            var result = new StringBuilder();

            result.AppendFormat(@"<button id='{0}' name='{0}' type='button' ", name);

            InjectAttributes(result, htmlAttributes);

            result.AppendFormat(@">{0}</button>", text);

            return new HtmlString(result.ToString());
        }

        private static void InjectAttributes(StringBuilder result, object htmlAttributes)
        {
            foreach (var item in new RouteValueDictionary(htmlAttributes))
            {
                result.AppendFormat(@"{0}='{1}' ", item.Key, item.Value);
            }
        }

        public static HtmlString Hidden<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return new HtmlString(html.Hidden(string.Format("{0}", formElement.ElementName), value).ToHtmlString());
        }

        public static HtmlString RawField<T>(this HtmlHelper html, FormElement<T> formElement, string contents)
        {
            return new HtmlString(string.Format("<div id='{0}-field' class='field'>{1}</div>", formElement, contents));
        }

        public static HtmlString Label<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return
                new HtmlString(string.Format("<label for='{0}'>{1}</label>", formElement.ElementName,
                                             formElement.LabelText(Resources.ResourceManager)));
        }

        public static HtmlString LabelledDropDownList<T>(this HtmlHelper html, FormElement<T> formElement,
                                                         SelectList list)
        {
            return html.RawField(formElement, html.RawLabelledDropDownList(formElement, list).ToString());
        }

        public static HtmlString RawLabelledDropDownList<T>(this HtmlHelper html, FormElement<T> formElement,
                                                            SelectList list)
        {
            return
                new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                             html.DropDownList(formElement.ElementName, list)));
        }

        public static HtmlString LabelledDropDownList<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return html.RawField(formElement, html.RawLabelledDropDownList(formElement).ToString());
        }

        public static HtmlString RawLabelledDropDownList<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return
                new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                             string.Format("<select id='{0}'></select>", formElement.ElementName)));
        }

        public static HtmlString LabelledListBox<T>(this HtmlHelper html, FormElement<T> formElement, SelectList list)
        {
            return html.RawField(formElement, html.RawLabelledListBox(formElement, list).ToString());
        }

        public static HtmlString RawLabelledListBox<T>(this HtmlHelper html, FormElement<T> formElement, SelectList list)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                html.ListBox(formElement.ElementName, list, new
                                                                                                {
                                                                                                    size = 5
                                                                                                })));
        }

        public static HtmlString LabelledDateBox<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawField(formElement, html.RawLabelledDateBox(formElement, value).ToString());
        }

        public static HtmlString RawLabelledDateBox<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawLabelledTextBox(formElement, value, new
                                                                   {
                                                                       Class = "date"
                                                                   });
        }

        public static HtmlString LabelledTextBox<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawField(formElement, html.RawLabelledTextBox(formElement, value).ToString());
        }

        public static HtmlString RawLabelledTextBox<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawLabelledTextBox(formElement, value, null);
        }

        public static HtmlString LabelledTextBox<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                    object htmlAttributes)
        {
            return html.RawField(formElement, html.RawLabelledTextBox(formElement, value, htmlAttributes).ToString());
        }

        public static HtmlString RawLabelledTextBox<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                       object htmlAttributes)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                html.TextBox(formElement.ElementName, value, htmlAttributes)));
        }

        public static HtmlString LabelledText<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawField(formElement, html.RawLabelledText(formElement, value).ToString());
        }

        public static HtmlString RawLabelledText<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawLabelledText(formElement, value, null);
        }

        public static HtmlString LabelledText<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                 object htmlAttributes)
        {
            return html.RawField(formElement, html.RawLabelledText(formElement, value).ToString());
        }

        public static HtmlString RawLabelledText<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                    object htmlAttributes)
        {
            var dictionary = new RouteValueDictionary(htmlAttributes);

            var css = string.Empty;

            const string CSS_KEY = "class";
            const string READ_ONLY_KEY = "readonly";

            if (dictionary.ContainsKey(CSS_KEY))
            {
                css = (string) dictionary[CSS_KEY];

                dictionary.Remove(CSS_KEY);
            }

            if (!css.Contains("text"))
            {
                css = string.Format("{0}{1}text", css, string.IsNullOrEmpty(css)
                                                           ? string.Empty
                                                           : " ");
            }

            dictionary.Add(CSS_KEY, css);

            if (!dictionary.ContainsKey(READ_ONLY_KEY))
            {
                dictionary.Add(READ_ONLY_KEY, READ_ONLY_KEY);
            }

            return new HtmlString(string.Format(@"<label for='{0}'>{1}{2}</label>", formElement.ElementName,
                                                formElement.LabelText(Resources.ResourceManager),
                                                html.TextBox(formElement.ElementName, value, dictionary)));
        }

        public static HtmlString LabelledTextArea<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return html.RawField(formElement, html.RawLabelledTextArea(formElement, value).ToString());
        }

        public static HtmlString RawLabelledTextArea<T>(this HtmlHelper html, FormElement<T> formElement, string value)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                html.TextArea(formElement.ElementName, value)));
        }

        public static HtmlString LabelledTextArea<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                     int rows, int cols)
        {
            return html.RawField(formElement, html.RawLabelledTextArea(formElement, value, rows, cols).ToString());
        }

        public static HtmlString RawLabelledTextArea<T>(this HtmlHelper html, FormElement<T> formElement, string value,
                                                        int rows, int cols)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                html.TextArea(formElement.ElementName, value, rows, cols, null)));
        }

        public static HtmlString LabelledPassword<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return html.RawField(formElement, html.RawLabelledPassword(formElement).ToString());
        }

        public static HtmlString RawLabelledPassword<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                html.Password(formElement.ElementName)));
        }

        public static HtmlString LabelledCheckBox<T>(this HtmlHelper html, FormElement<T> formElement,
                                                     bool? value)
        {
            return LabelledCheckBox(html, formElement, value, null);
        }

        public static HtmlString LabelledCheckBox<T>(this HtmlHelper html, FormElement<T> formElement,
                                                     bool? value, object htmlAttributes)
        {
            return html.RawField(formElement, html.RawLabelledCheckBox(formElement, value, htmlAttributes).ToString());
        }

        public static HtmlString RawLabelledCheckBox<T>(this HtmlHelper html, FormElement<T> formElement,
                                                        bool? value, object htmlAttributes)
        {
            return new HtmlString(string.Format(@"{0}{1}", html.Label(formElement),
                                                formElement.LabelText(Resources.ResourceManager)));
        }

        public static HtmlString RawCheckBox(this HtmlHelper html, string name, bool @checked)
        {
            return RawCheckBox(html, name, @checked, null);
        }

        public static HtmlString RawCheckBox(this HtmlHelper html, string name, bool @checked, object htmlAttributes)
        {
            var result = new StringBuilder();

            result.AppendFormat(@"<input type='checkbox' id='{0}' name='{0}'{1} value='true'", name,
                                (@checked
                                     ? " checked"
                                     : string.Empty));

            InjectAttributes(result, htmlAttributes);

            result.Append(@"/>");

            return new HtmlString(result.ToString());
        }

        public static HtmlString HelpLink(this HtmlHelper html, string key)
        {
            return HelpLink(html, key, null);
        }

        public static HtmlString HelpLink(this HtmlHelper html, string key, string title)
        {
            return
                new HtmlString(string.Format("<a class='help no-wait' key='{0}'{1}>help</a>", key,
                                             string.IsNullOrEmpty(title)
                                                 ? string.Empty
                                                 : string.Format(
                                                     " title='{0}'", title)));
        }

        public static HtmlString CssInclude(this HtmlHelper html, params string[] urls)
        {
            var sb = new StringBuilder();

            foreach (var url in urls)
            {
                sb.AppendLine(string.Format("<link rel=\"Stylesheet\" type=\"text/css\" href=\"{0}?{1}\" />",
                                            new UrlHelper(html.ViewContext.RequestContext).Content(url), Version()));
            }

            return new HtmlString(sb.ToString());
        }

        public static HtmlString ResolveUrl(this HtmlHelper html, string url)
        {
            return new HtmlString(new UrlHelper(html.ViewContext.RequestContext).Content(url));
        }

        public static HtmlString JavascriptInclude(this HtmlHelper html, string file)
        {
            return JavascriptInclude(html, file, new object[] {});
        }

        public static HtmlString JavascriptInclude(this HtmlHelper html, string url, params object[] args)
        {
            return
                new HtmlString(
                    string.Format("<script language=\"javascript\" type=\"text/javascript\" src=\"{0}?{1}\"></script>",
                                  new UrlHelper(html.ViewContext.RequestContext).Content(string.Format(url, args)),
                                  Version()));
        }

        public static ContentForm ContentForm(this HtmlHelper html, UrlHelper url, FormModel model)
        {
            return ContentForm(html, url.Action(model.Action, model.Controller, model.RouteValues),
                               model.SubmitButtonText, string.Empty);
        }

        public static ContentForm ContentForm(this HtmlHelper html, UrlHelper url, FormModel model,
                                              string containerClass)
        {
            return ContentForm(html, url.Action(model.Action, model.Controller, model.RouteValues),
                               model.SubmitButtonText, containerClass);
        }

        public static ContentForm ContentForm(this HtmlHelper html, string action)
        {
            return ContentForm(html, action, string.Empty, string.Empty);
        }

        public static ContentForm ContentForm(this HtmlHelper html, string action, string submitButtonText,
                                              string containerClass)
        {
            Guard.AgainstNullOrEmptyString(action, "action");

            var output = new StringBuilder();

            output.AppendLine(string.Format("<div class='content-form-container {0}'>",
                                            (string.IsNullOrEmpty(containerClass)
                                                 ? string.Empty
                                                 : containerClass)));
            output.AppendLine(
                string.Format(
                    "<form id='PropertiesForm' name='PropertiesForm' action='{0}' method='post' class='content-form'>",
                    action));

            html.ViewContext.Writer.Write(output.ToString());

            return new ContentForm(html.ViewContext,
                                   Button(html, "button-submit", !string.IsNullOrEmpty(submitButtonText)
                                                                     ? submitButtonText
                                                                     : "Submit"));
        }

        public static EndTag FormField<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            html.ViewContext.Writer.Write(string.Format("<div id='{0}-field' class='field'>", formElement));

            return new EndTag(html.ViewContext, "</div>");
        }

        public static string RenderPartialViewToString(this Controller controller, string viewName, object model)
        {
            controller.ViewData.Model = model;
            try
            {
                using (var sw = new StringWriter())
                {
                    var viewResult = ViewEngines.Engines.FindPartialView(controller.ControllerContext, viewName);
                    var viewContext = new ViewContext(controller.ControllerContext, viewResult.View, controller.ViewData, controller.TempData, sw);
                    viewResult.View.Render(viewContext, sw);

                    return sw.GetStringBuilder().ToString();
                }
            }
            catch (Exception ex)
            {
                return ex.ToString();
            }
        }

        public static HtmlString LabelledFileInput<T>(this HtmlHelper html, FormElement<T> formElement)
        {
            return LabelledFileInput(html, formElement, null);
        }

        public static HtmlString LabelledFileInput<T>(this HtmlHelper html, FormElement<T> formElement,
                                                      object htmlAttributes)
        {
            return new HtmlString(string.Format(@"<label for='{0}'>{1}{2}</label>", formElement.ElementName,
                                                formElement.LabelText(Resources.ResourceManager),
                                                string.Format(
                                                    "<input id='{0}' name='{0}' class='file-input ui-helper-hidden' type='file' />",
                                                    formElement.ElementName)));
        }
    }
}