using System.Collections.Generic;
using System.Web.Mvc;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public static class DataDictionaryExtensions
    {
        private const string FailureMessagesKey = "FailureMessages";
        private const string SuccessMessagesKey = "SuccessMessages";

		public static void Add<T>(this TempDataDictionary data, T value)
		{
			var key = typeof(T).FullName;

			if (data.ContainsKey(key))
			{
				data.Remove(key);
			}

			data.Add(key, value);
		}

		public static T Get<T>(this TempDataDictionary data)
		{
			var key = typeof (T).FullName;

			if (data.ContainsKey(key))
			{
				return (T)data[key];
			}

			return default(T);
		}

    	public static IList<string> GetFailureMessages(this TempDataDictionary data)
        {
            if (data.ContainsKey(FailureMessagesKey))
            {
                return (IList<string>)data[FailureMessagesKey];
            }

            return new List<string>();
        }

        public static IList<string> GetSuccessMessages(this TempDataDictionary data)
        {
            if (data.ContainsKey(SuccessMessagesKey))
            {
                return (IList<string>)data[SuccessMessagesKey];
            }

            return new List<string>();
        }

        public static void SetFailureMessages(this TempDataDictionary data, params string[] failureMessages)
        {
            SetFailureMessages(data, (IEnumerable<string>)failureMessages);
        }

        public static void SetFailureMessages(this TempDataDictionary data, IEnumerable<string> failureMessages)
        {
            if (data.ContainsKey(FailureMessagesKey))
            {
                data.Remove(FailureMessagesKey);
            }

            data.Add(FailureMessagesKey, new List<string>(failureMessages));
        }

        public static void SetSuccessMessages(this TempDataDictionary data, params string[] successMessages)
        {
            SetSuccessMessages(data, (IEnumerable<string>)successMessages);
        }

        public static void SetSuccessMessages(this TempDataDictionary data, IEnumerable<string> successMessages)
        {
            if (data.ContainsKey(SuccessMessagesKey))
            {
                data.Remove(SuccessMessagesKey);
            }

            data.Add(SuccessMessagesKey, new List<string>(successMessages));
        }

        public static bool HasFailureMessages(this TempDataDictionary data)
        {
            return data.ContainsKey(FailureMessagesKey) && ((IList<string>)data[FailureMessagesKey]).Count > 0;
        }

        public static bool HasSuccessMessages(this TempDataDictionary data)
        {
            return data.ContainsKey(SuccessMessagesKey) && ((IList<string>)data[SuccessMessagesKey]).Count > 0;
        }

        public static void SetResultMessages(this TempDataDictionary data, IResult result)
        {
            if (result.HasSuccessMessages)
            {
                SetSuccessMessages(data, result.SuccessMessages.ConvertAll(item => item.ToString()));
            }

            if (result.HasFailureMessages)
            {
                SetFailureMessages(data, result.FailureMessages.ConvertAll(item => item.ToString()));
            }
        }
    }
}